/**
    @file ft51_uart.h

    @brief UART Driver

    Simple blocking read and write functions, plus basic
    configuration.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_UART_H__
#define __FT51_UART_H__

#include <stdint.h>
#include "ft51.h"



/**
    @enum  UART_DATA_BITS
    @brief Number of data bits in UART characters.
**/
typedef enum
{
    /** Characters are 7 bits long. Can transmit all standard ASCII codes. 
        Cannot send characters greater than 127(dec) or any binary data. **/
    UART_DATA_BITS_7    = 7,        
    /** Characters are 8 bits long. Can transmit all data including binary 
        data. (Default) **/
    UART_DATA_BITS_8    = 8,        
} UART_DATA_BITS;

/**
    @enum  UART_STOP_BITS
    @brief Number of stop bits framing a UART character.
**/
typedef enum
{
    /** 1 Stop bit sent after a UART character. (Default) **/
    UART_STOP_BITS_1    = 1,        
    /** 2 Stop bits sent after a UART character. **/
    UART_STOP_BITS_2    = 2,
} UART_STOP_BITS;

/**
    @enum  UART_PARITY
    @brief Parity bit used in UART characters. This may be disabled resulting 
        in no parity bit being sent.
        One parity bit may be sent after the UART character. This bit can be 
        used for error detection when set to odd or even.
**/
typedef enum
{
    /** Odd parity notes that an odd number of state transitions has occurred 
        in the UART character. **/
    UART_PARITY_ODD     = 0,
    /** Even parity notes that an even number of state transitions has occurred 
        in the UART character. **/
    UART_PARITY_EVEN    = 1, 
    /** Parity bit is always a logic '1'. **/
    UART_PARITY_MARK    = 2, 
    /** Parity bit is always a logic '0'. **/
    UART_PARITY_SPACE   = 3, 
    /** No parity bit is sent. No parity bit is expected to be received. **/
    UART_PARITY_NONE    = 4
} UART_PARITY;

/**
    @enum  UART_FLOW_CONTROL
    @brief Flow control method used for UART. If this is enabled then the 
        receiver can request that the transmitting device stop transmission 
        until the receiver is ready.
**/
typedef enum
{
    /** No flow control enabled. RTS and DTR signals, and XOFF characters 
        ignored. **/
    UART_FLOW_NONE       = 0,
    /** Hardware flow control. Uses RTS output to indicate to the transmitter 
        to stop sending data. CTS is input which tells the UART to stop sending 
        data to the remote receiver. **/
    UART_FLOW_RTS_CTS    = 1,
    /** Hardware flow control. Uses DTR output to indicate to the transmitter 
        to stop sending data. DSR is input which tells the UART to stop sending 
        data to the remote receiver. **/
    UART_FLOW_DTR_DSR    = 2
} UART_FLOW_CONTROL;



/* Function Prototypes */

/**
    @brief      Initialise UART hardware.

    Performs a software reset and enables the UART hardware.  This 
    function MUST be called before any call to the other UART 
    functions.
**/
void UART_initialise(void);

/**
    @brief      Receive bytes arriving at the FT51 UART interface.


    Read the specified number of bytes into the given buffer.
    This function will block until all data has been received.
    @param[out] data    buffer with space for data to be read from the UART.
    @param[in]  length  number of bytes to be read from the UART.
**/
void UART_read(uint8_t        *data,
               const uint16_t  length);

/**
    @brief      Transmit bytes from the FT51 UART interface.


    Write the specified number of bytes from the given buffer to 
    the UART.  This function will block until all data has reached 
    the UART.
    @param[in]  data     buffer containing the data to be written to the UART.
    @param[in]  length   number of bytes to be written to the UART.
**/
void UART_write(uint8_t        *data,
                const uint16_t  length);

/**
    @brief      Set UART baud rate.


    Sets the UART baud rate.
    The function reads the current system clock frequency to
    calculate divisors for the requested baud rate.
    If the system clock frequency is changed then the baud rate
    must be set again to ensure correct operation.
    @note       The default baud rate is 9600 for 48 MHz clock.
    @param[in]  baud_rate    Requested UART baud rate.  The actual baud
                             rate will be this figure rounded down to the
                             nearest rate which exactly divides the system 
                             clock frequency.  The maximum supported rate
                             is 3 million.
**/
void UART_set_baud_rate(uint32_t baud_rate);

/**
    @brief      Set UART flow control mode.

    Select the flow control method to use on the UART.
    @param[in]  flow_control  Either RTS/CTS, DTR/DSR, or None.
**/
void UART_set_flow_control (UART_FLOW_CONTROL flow_control);

/**
    @brief      Set number of UART data bits.

    Select either 7 or 8 data bits per frame.  The default is 8.
    @param[in]  data_bits    Member of UART_DATA_BITS enum.
**/
void UART_set_data_bits (UART_DATA_BITS data_bits);

/**
    @brief      Set number of UART stop bits.

    Select either 1 or 2 stop bits per frame.  The default is 1.
    @param[in]  stop_bits    Member of UART_STOP_BITS enum.
**/
void UART_set_stop_bits (UART_STOP_BITS stop_bits);

/**
    @brief      Set UART parity mode.

    Select whether or not to include (or expect) a parity bit 
                in each frame sent (or received), and the scheme used to
                generate (or interpret) such a bit.  By default, no parity
                bit is sent or expected.
    @param[in]  parity  Member of UART_PARITY enum.
**/
void UART_set_parity (UART_PARITY parity);

#endif // __FT51_UART_H__